﻿/*****************************************************
 * Project: Virtual Campus Tour
 * Programmer: Drew Nackers 
 * Assistance: Dr. Mc Vey, Dr. Pankratz, Mr. Blahnik 
 * Date: Spring Semester 2009
 * Description: This dialog displays information about Cofrin hall. 
 * This dialog can be opened either manually by clicking on the 
 * interactive map or automatically by entering a GPS box around
 * Cofrin Hall.
 * ***************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX.AudioVideoPlayback;

namespace VirtualCampus
{
    public partial class Cofrin : Form
    {
        public bool COF
        {
            get {return cofopen;} // returns that the dialog is currently open
        }
        // gets new video
        private Video vid;
        bool cofopen = true;

        public Cofrin()
        {

            InitializeComponent();
            
            // sets up the video and plays it
            SetUpVideo();
        }

        // creates an event that updates the status of the dialog back to the main dialog
        public event EventHandler EnableNewDialog;

        /****************************************************
        * Name:SetUpVideo
        * Discription: Loads video, sets up a box for it to 
        * play in, and plays the video.
        * **************************************************/
        private void SetUpVideo()
        {
            // stores videobox's original sizes
            int height = videobox.Height;
            int width = videobox.Width;

            //loads video file
            vid = new Video(@"Cofrin3.avi");
            vid.Owner = videobox;

            // restores videobox's size to original
            videobox.Width = width;
            videobox.Height = height;

            // plays when box is open
            vid.Play();
        }

        /****************************************************
        * Name:playbtn_Click
        * Discription: Handles click on the play button
        * **************************************************/
        private void playbtn_Click(object sender, EventArgs e)
        {
            vid.Play();
        }

        /****************************************************
        * Name:stopbtn_Click
        * Discription: Handles click on the stop button.
        * **************************************************/
        private void stopbtn_Click(object sender, EventArgs e)
        {
            vid.Stop();
        }

        /****************************************************
        * Name:pausebtn_Click
        * Discription: Handles click on the pause button.
        * **************************************************/
        private void pausebtn_Click(object sender, EventArgs e)
        {
            vid.Pause();
        }

        /****************************************************
        * Name:exitToolStripMenuItem_Click
        * Discription: Stops the video and closes the dialog
        * **************************************************/
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            vid.Stop();
            Close();
        }

        /****************************************************
        * Name:ThumbOneEnter
        * Discription: If mouse over thumbnail a large picture
        * is displayed
        * **************************************************/
        private void ThumbOneEnter(object sender, EventArgs e)
        {
            Image newpic;
            newpic = new Bitmap(@"Cofrin.jpg");
            largepicture.Image = newpic;
        }

        // mouse over anywhere on main screen
        private void MainMenuOver(object sender, EventArgs e)
        {
           // Image newpic;
           // newpic = new Bitmap(@"C:\Documents and Settings\Administrator\My Documents\Capstone\VirtualCampus\Photos\Cofrin\Large\L_Intro.bmp");
           // largepicture.Image = newpic;
        }

        /****************************************************
        * Name:ThumbTwoEnter
        * Discription: If mouse over thumbnail a large picture
        * is displayed
        * **************************************************/
        private void ThumbTwoEnter(object sender, EventArgs e)
        {
            Image newpic;
            newpic = new Bitmap(@"CofrinHall.jpg");
            largepicture.Image = newpic;
        }

        /****************************************************
        * Name:ThumbThreeEnter
        * Discription: If mouse over thumbnail a large picture
        * is displayed
        * **************************************************/
        private void ThumbThreeEnter(object sender, EventArgs e)
        {
            Image newpic;
            newpic = new Bitmap(@"CofrinClass.jpg");
            largepicture.Image = newpic;
        }

        /****************************************************
        * Name:ThumbFourEnter
        * Discription: If mouse over thumbnail a large picture
        * is displayed
        * **************************************************/
        private void ThumbFourEnter(object sender, EventArgs e)
        {
            // declares new image 
            Image newpic;

            // puts in new file path
            newpic = new Bitmap(@"cofrincomputerlab.jpg");
            
            // places thumbnail in larger box
            largepicture.Image = newpic;
        }

        /****************************************************
        * Name:Cofrin_FormClosing
        * Discription: When the form is closing it stops the video
        * and resets the bool variable to false so the dialog can
        * open agian
        * **************************************************/
        private void Cofrin_FormClosing(object sender, FormClosingEventArgs e)
        {
            vid.Stop();
            if (EnableNewDialog != null)
                EnableNewDialog(this, new EventArgs());
            // some how set cofopen back to false so dialog can open other ways
            
        }

    }
}
