﻿/*****************************************************
 * Project: Virtual Campus Tour
 * Programmer: Drew Nackers 
 * Assistance: Dr. Mc Vey, Dr. Pankratz, Mr. Blahnik 
 * Date: Spring Semester 2009
 * Description: This program reads in raw NMEA strings through a comport using a GPS usb stick.
 * Everytime a new reading occurs the comport fires an event to handle the new data.  From there
 * the data is read into a byte array where the first 6 bytes are checked for the correct header
 * using a switch case.  If it passes through all the cases we have the string we are looking for
 * and it puts the remaining bytes into a string.  These remaining bytes contain the latitude and 
 * longitude readings.  Once we have the current GPS readings several things are done with them. 
 * The first thing is looking to see if we are near a current building.  This is done through if
 * statements and GPS readings taken of the buildings. If a user is inside a GPS box around one 
 * of the buildings a dialog with information about that building will open that contains video,
 * audio, and pictures about the current building they are viewing. Another thing that the readings
 * are used for are the computation of distances to buildings from a users current point this can be
 * viewed by clicking the distance menu item under location menu.  The final thing that the readings 
 * are used for are displaying the users current location which can be viewed by clicking the current
 * location menu item under the location menu. The program also has a "non-gps" portion.  A user 
 * may choose to not connect to the GPS and they are still able to access various dialogs with 
 * information about by either clicking on the interactive map or choosing a building from the tool 
 * strip.
 * ***************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VirtualCampus
{
    public partial class virtualtour : Form
    {
        //enumerated type for each state of switch case
        public enum State
        {
            Dollar_State = 0,
            G_State,
            P_State,
            GG_State,
            GGG_State,
            A_State,
            Data_State,
        };

        // used to store the current state of switch case
        private State CurState;

        // variables used by switch case
        int index = 0;
        byte[] data = new byte[256];

        // variables to store latitude and longitude
        double latmin = 0.0;
        public double latdeg = 0.0;
        double latval = 0.0;
        double longmin = 0.0;
        public double longdeg = 0.0;
        double longval = 0.0;

        // bools for dialog being open or not
        bool cof = false;
        bool jmss = false;
        bool bem = false;

        // default mouse cords
        int mousex = 0;
        int mousey = 0;

        //string for comport
        string port = "";

        public virtualtour()
        {
            InitializeComponent();

            DisconnectToolStripMenuItem.Enabled = false; 
            infoboxLabel.Text = "Disconnected"; // Status of GPS starts out disconnected
        }
        /****************************************************
         * Name:cofrinToolStripMenuItem_Click
         * Discription: Handles a click on Cofrin Hall through the 
         * menu strip and displays it's dialog with infromation
         * about Cofrin hall. 
         * **************************************************/
        private void cofrinToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // if menu item clicked the Cofrin Hall dialog is displayed
            if (cof == false)
            {
                Cofrin coftool = new Cofrin();
                cof = coftool.COF;
                // this event handler allows the the bool for the dialog to go back to false
                // so it can reopen again after it is closed
                coftool.EnableNewDialog += new EventHandler(EnableNewDialog);
                coftool.Show();
            }
        }

        /****************************************************
        * Name:exitToolStripMenuItem_Click
        * Discription: Closes the current dialog
        * **************************************************/
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // closes form
            Close();
        }

        /****************************************************
        * Name:comport_DataReceived
        * Discription: New data received in comport.  The 
        * data is put through a switch case to determine if
        * it is the string we are looking for, if it is 
        * the information is placed into a string.
        * **************************************************/
        private void comport_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            // byte array of the size of bytes in the comport so space is not wasted
            byte[] buf = new byte[comport.BytesToRead];
            
            // reads what is in the comport and places it in the buffer
            comport.Read(buf, 0, buf.Length);

            // looks at each byte in the buffer
            foreach (byte b in buf)
            {
                switch (CurState)
                {
                    case State.Dollar_State:
                        if (b == '$') // first char in the string
                        {
                            index = 0;
                            CurState = State.G_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; // start over
                        break;

                    case State.G_State:
                        if (b == 'G') // second char in the string
                        {
                            CurState = State.P_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; // start over
                        break;
                    case State.P_State:
                        if (b == 'P') // third char in the string
                        {
                            CurState = State.GG_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; // start over
                        break;
                    case State.GG_State:
                        if (b == 'G') // fourth char in the string
                        {
                            CurState = State.GGG_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; // start over
                        break;
                    case State.GGG_State:
                        if (b == 'G') // fifth char in the string
                        {
                            CurState = State.A_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; //start over
                        break;
                    case State.A_State:
                        if (b == 'A') // sixth char in the string
                        {
                            CurState = State.Data_State; // next state
                        }
                        else
                            CurState = State.Dollar_State; // start over
                        break;
                    case State.Data_State:
                        if (b != '\r') // if it is not the end of the array
                        {
                            data[index] = b; // place byte in array
                            index++; // next spot in array
                            CurState = State.Data_State; // loop
                        }
                        else
                        {
                            // convert byte array into string
                            string GoodData = System.Text.ASCIIEncoding.GetEncoding(0).GetString(data);
                            parsedata(GoodData); // calls function to parse out data
                            CurState = State.Dollar_State; // starts over
                        }
                        break;
                    default:
                        CurState = State.Dollar_State;  // default is first state
                        break;
                }
            }
        }

        /****************************************************
        * Name:parsedata
        * Discription: The NEMA string is passed to this function
        * from the comport recieved event.  Here the string is 
        * parsed out into useful data like latitude and longitude.
        * Then that data is placed in variables.
        * **************************************************/
        private void parsedata(string NEMAData)
        {
            // splits on comma and puts into an array
            string[] parstr = NEMAData.Split(',');

            // must have this or quits on start up with a blank string
            if (parstr[2] != "" || parstr[4] != "")
            {
                // converts lat and long to deg.decimaldeg
                latval = Convert.ToDouble(parstr[2]);
                longval = Convert.ToDouble(parstr[4]);

                latdeg = (int)(latval / 100.0);
                longdeg = (int)(longval / 100.0);

                latmin = latval - (latdeg * 100.0);
                longmin = longval - (longdeg * 100.0);

                latdeg = latdeg + (latmin / 60.0);
                longdeg = longdeg + (longmin / 60.0);
            }

            if(latdeg == 0 && longdeg == 0)
            {
                // if there is no fix and the comport is open
                if (comport.IsOpen == true)
                {
                    infoboxLabel.Text = "Searching";
                }
            }
            else
                infoboxLabel.Text = "Connected";

        }

        /****************************************************
        * Name:cofrindialog
        * Discription: The current latitude and longitude is
        * passed to this function.  They are used to check and
        * see if a user is inside that building. If they are
        * the dialog about that building is displayed.
        * **************************************************/
        private void cofrindialog(double latdeg, double longdeg)
        {
            // latitude box
            if (latdeg < 44.446166 && latdeg > 44.44565)
            {
                //longitude box
                if (longdeg < 88.067816 && longdeg > 88.0670616)
                {
                    // if dialog isn't already opened
                    if (cof == false)
                    {
                        Cofrin GPSCof = new Cofrin();
                        cof = GPSCof.COF;
                        GPSCof.EnableNewDialog += new EventHandler(EnableNewDialog);
                        GPSCof.Show();
                    }
                }
            }
            
        }

        /****************************************************
        * Name:jmsdialog
        * Discription: The current latitude and longitude is
        * passed to this function.  They are used to check and
        * see if a user is inside that building. If they are
        * the dialog about that building is displayed.
        * **************************************************/
        private void jmsdialog(double latdeg, double longdeg)
        {
            // latitude box
            if (latdeg < 44.446096 && latdeg > 44.4452716)
            {
                //longitude box
                if (longdeg < 88.067095 && longdeg > 88.0665383)
                {
                    // if dialog isn't already opened
                    if (jmss == false)
                    {
                        jms GPSjms = new jms();
                        jmss = GPSjms.JMS;
                        GPSjms.EnableNewDialog += new EventHandler(EnableNewDialog);
                        GPSjms.Show();
                    }
                }
            }

        }

        /****************************************************
        * Name:bemdialog
        * Discription: The current latitude and longitude is
        * passed to this function.  They are used to check and
        * see if a user is inside that building. If they are
        * the dialog about that building is displayed.
        * **************************************************/
        private void bemdialog(double latdeg, double longdeg)
        {
            // latitude box
            if (latdeg < 44.4456283 && latdeg > 44.4451583)
            {
                //longitude box
                if (longdeg < 88.06846 && longdeg > 88.067696)
                {
                    // if dialog isn't already opened
                    if (bem == false)
                    {
                        bemis GPSbem = new bemis();
                        bem = GPSbem.BEM;
                        GPSbem.EnableNewDialog += new EventHandler(EnableNewDialog);
                        GPSbem.Show();
                    }
                }
            }

        }


        /****************************************************
        * Name:ConnectToolStripMenuItem_Click
        * Discription: Handles a click on connect through the 
        * menu strip and connects the GPS device
        * **************************************************/
        private void ConnectToolStripMenuItem_Click(object sender, EventArgs e)
        { 
            ConnectToolStripMenuItem.Enabled = false;
            DisconnectToolStripMenuItem.Enabled = true;
            comport.Open(); // opens comport

            // display label in bttom corner saying connected
            infoboxLabel.Text = "Connected";
            CheckCords_Timer.Start(); // starts timer to check cords
        }

        /****************************************************
       * Name:DisconnectToolStripMenuItem_Click
       * Discription: Handles a click on disconnect through the 
       * menu strip and disconnects the GPS device
       * **************************************************/
        private void DisconnectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConnectToolStripMenuItem.Enabled = true;
            DisconnectToolStripMenuItem.Enabled = false;
            comport.Close(); // closes comport

            // display label in bttom corner saying disconnected
            infoboxLabel.Text = "Disconnected";
            CheckCords_Timer.Stop(); // starts timer to check cords

            // clears lat and long after close to prevent dialogs from opening
            // after the program is closed, if you closed while in a building
            latdeg = 0;
            longdeg = 0;
        }

        /****************************************************
       * Name:comport_ErrorReceived
       * Discription: Displays connection error in GPS status
       * if the comport encounters an error
       * **************************************************/
        private void comport_ErrorReceived(object sender, System.IO.Ports.SerialErrorReceivedEventArgs e)
        {
            // display label in bottom corner saying error
            infoboxLabel.Text = "Connection Error";
        }

        /****************************************************
       * Name:currentLocationToolStripMenuItem_Click
       * Discription: Handles a click on current location through the 
       * menu strip and displays the current latitude and longitude.
       * **************************************************/
        private void currentLocationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // open dialog with current lat and long positions 
            cords corddlg = new cords();
            corddlg.Latt = latdeg.ToString();
            corddlg.Longg = longdeg.ToString();
            corddlg.Show();
        }

        /****************************************************
       * Name:Mouse_Move
       * Discription: Handles any mouse movement on the screen. 
       * In this case it checks to see if the pointer is over
       * a building of interest and displays the name of that
       * building on the map.
       * NOTE: JMS needs three boxes because of its odd shape
       * **************************************************/
        private void Mouse_Move(object sender, MouseEventArgs e)
        {
            mousex = e.X;
            mousey = e.Y;

            // cofrin mouse over
            if (mousex >= 226 && mousex <= 250)
            {
                if (mousey >= 313 && mousey <= 360)
                {
                    cofmaplabel.Text = "Cofrin";
                }
                else
                    cofmaplabel.Text = "";
            }
            else
                cofmaplabel.Text = "";

            // bemis mouse over
            if (mousex >= 252 && mousex <= 297)
            {
                if (mousey >= 358 && mousey <= 387)
                {
                    bemismaplabel.Text = "Bemis";
                }
                else
                    bemismaplabel.Text = "";
            }
            else
                bemismaplabel.Text = "";

            // jms box one
            if (mousex >= 230 && mousex <= 254)
            {
                if (mousey >= 218 && mousey <= 289)
                {
                    jmsmaplebel.Text = "JMS";
                }
                else
                    jmsmaplebel.Text = "";
            }

            //jms box two
            else if (mousex >= 254 && mousex <= 275)
            {
                if (mousey >= 250 && mousey <= 290)
                {
                    jmsmaplebel.Text = "JMS";
                }
                else
                    jmsmaplebel.Text = "";
            }

            //jms box three
            else if (mousex >= 275 && mousex <= 290)
            {
                if (mousey >= 270 && mousey <= 290)
                {
                    jmsmaplebel.Text = "JMS";
                }
                else
                    jmsmaplebel.Text = "";
            }
            else
                jmsmaplebel.Text = "";
            
        }

        /****************************************************
       * Name:virtualtour_MouseClick
       * Discription: If a box is click on the map
       * the dialog about that building is displayed
       * NOTE: JMS has three boxes because of its odd shape
       * **************************************************/
        private void virtualtour_MouseClick(object sender, MouseEventArgs e)
        {
            // box of x & y cords for cofrin
            if (mousex >= 226 && mousex <= 250)
            {
                if (mousey >= 313 && mousey <= 360)
                {
                    if (cof == false)
                    {
                        Cofrin clickCof = new Cofrin();
                        cof = clickCof.COF;
                        clickCof.EnableNewDialog += new EventHandler(EnableNewDialog);
                        clickCof.Show();
                    }
                }
            }

            // box of x & y cords for bemis
            if (mousex >= 252 && mousex <= 297)
            {
                if (mousey >= 358 && mousey <= 387)
                {
                    if (bem == false)
                    {
                        bemis clickBem = new bemis();
                        bem = clickBem.BEM;
                        clickBem.EnableNewDialog += new EventHandler(EnableNewDialog);
                        clickBem.Show();
                    }
                }
            }

            // // box of x & y cords for jms box one
            if (mousex >= 230 && mousex <= 254)
            {
                if (mousey >= 218 && mousey <= 289)
                {
                    if (jmss == false)
                    {
                        jms clickJms1 = new jms();
                        jmss = clickJms1.JMS;
                        clickJms1.EnableNewDialog += new EventHandler(EnableNewDialog);
                        clickJms1.Show();
                    }
                }
            }

            //// box of x & y cords for jms box two
            else if (mousex >= 254 && mousex <= 275)
            {
                if (mousey >= 250 && mousey <= 290)
                {
                    if (jmss == false)
                    {
                        jms clickJms2 = new jms();
                        jmss = clickJms2.JMS;
                        clickJms2.EnableNewDialog += new EventHandler(EnableNewDialog);
                        clickJms2.Show();
                    }
                }
            }

            //// box of x & y cords for jms box three
            else if (mousex >= 275 && mousex <= 290)
            {
                if (mousey >= 270 && mousey <= 290)
                {
                    if (jmss == false)
                    {
                        jms clickJms3 = new jms();
                        jmss = clickJms3.JMS;
                        clickJms3.EnableNewDialog += new EventHandler(EnableNewDialog);
                        clickJms3.Show();
                    }
                }
            }
        }

        /****************************************************
       * Name:CofLabelClick
       * Discription: The user clicks the label on a building.
       * The dialog for that building is displayed
       * **************************************************/
        private void CofLabelClick(object sender, EventArgs e)
        {
            if (cof == false)
            {
                Cofrin clickCofLabel = new Cofrin();
                cof = clickCofLabel.COF;
                clickCofLabel.EnableNewDialog += new EventHandler(EnableNewDialog);
                clickCofLabel.Show();
            }
        }

        /****************************************************
        * Name:BemisLabelClick
        * Discription: The user clicks the label on a building.
        * The dialog for that building is displayed
        * **************************************************/
        private void BemisLabelClick(object sender, EventArgs e)
        {
            if (bem == false)
            {
                bemis clickBemLabel = new bemis();
                bem = clickBemLabel.BEM;
                clickBemLabel.EnableNewDialog += new EventHandler(EnableNewDialog);
                clickBemLabel.Show();
            }
        }

        /****************************************************
        * Name:JMSLabelClick
        * Discription: The user clicks the label on a building.
        * The dialog for that building is displayed
        * **************************************************/
        private void JMSLabelClick(object sender, EventArgs e)
        {
            if (jmss == false)
            {
                jms clickJmsLabel = new jms();
                jmss = clickJmsLabel.JMS;
                clickJmsLabel.EnableNewDialog += new EventHandler(EnableNewDialog);
                clickJmsLabel.Show();
            }
        }

        /****************************************************
        * Name:buildingDiToolStripMenuItem_Click
        * Discription: Handles a click on building distance through the 
        * menu strip and displays the current distance to Cofrin, JMS and Bemis.
        * **************************************************/
        private void buildingDiToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // open dialog with distances to buildings from current locations
            distance distdlg = new distance(latdeg,longdeg);
            distdlg.CurLat = latdeg;
            distdlg.CurLong = longdeg;
            distdlg.Show();
        }

        /****************************************************
        * Name:jMSToolStripMenuItem_Click
        * Discription: Handles a click on JMS through the 
        * menu strip and displays it's dialog with infromation
        * about JMS. 
        * **************************************************/
        private void jMSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (jmss == false)
            {
                jms jmstool = new jms();
                jmss = jmstool.JMS;
                jmstool.EnableNewDialog += new EventHandler(EnableNewDialog);
                jmstool.Show();
            }
        }

        /****************************************************
        * Name:bemisToolStripMenuItem_Click
        * Discription: Handles a click on Bemis through the 
        * menu strip and displays it's dialog with infromation
        * about Bemis. 
        * **************************************************/
        private void bemisToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (bem == false)
            {
                bemis bemtool = new bemis();
                bem = bemtool.BEM;
                bemtool.EnableNewDialog += new EventHandler(EnableNewDialog);
                bemtool.Show();
            }
        }

      /****************************************************
      * Name:EnableNewDialog
      * Discription: This event is handled when a building dialog
      * is displayed and then closed.  This resets the current 
      * status of that building back to false so it may open agian.
      * **************************************************/
        private void EnableNewDialog(object src, EventArgs e)
        {
            cof = false;
            jmss = false;
            bem = false;
        }

        /****************************************************
        * Name:CheckBuildings_Tick
        * Discription: This event is handled every tick of the 
        * the timer.  It checks each buildings latitude and
        * longitude box to see if the current latitude and 
        * longitude is within any of the boxes.
        * **************************************************/
        private void CheckBuildings_Tick(object sender, EventArgs e)
        {
            // function calls for buildings
            cofrindialog(latdeg, longdeg);
            jmsdialog(latdeg, longdeg);
            bemdialog(latdeg, longdeg);
            
        }

        /****************************************************
       * Name:comportToolStripMenuItem_Click
       * Discription: This event is handles a click on the 
       * change comport menu item and opens a dialog to 
       * change the comport of the gps.
       * **************************************************/
        private void comportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            comport comportchange = new comport();
            port = comportchange.PORT;
            comportchange.changeport += new EventHandler(changeport);
            comportchange.Show();
        }

        /****************************************************
       * Name:changeport
       * Discription: This event is fired when the comport
       * dialog is closed. It updates the port number of
       * the gps.
       * **************************************************/
        private void changeport(object src, EventArgs e)
        {
            // closes old comport to prevent errors with multiple comports
            ConnectToolStripMenuItem.Enabled = true;
            DisconnectToolStripMenuItem.Enabled = false;
            comport.Close(); // closes comport
            infoboxLabel.Text = "Disconnected";
            //resets lat long
            latdeg = 0;
            longdeg = 0;
            comport.PortName = src.ToString(); // changes port number 
        }
    }
}
